@@ -17,18 +17,20 @@
 
 """TXT-like base class."""
 
-import struct
-from typing import Any, Dict, Iterable, Optional, Tuple, Union
+from typing import Any, Dict, Iterable, Tuple
 
 import dns.exception
 import dns.immutable
+import dns.name
 import dns.rdata
+import dns.rdataclass
+import dns.rdatatype
+import dns.renderer
 import dns.tokenizer
 
 
 @dns.immutable.immutable
 class TXTBase(dns.rdata.Rdata):
-
     """Base class for rdata that is like a TXT record (see RFC 1035)."""
 
     __slots__ = ["strings"]
@@ -37,7 +39,7 @@ class TXTBase(dns.rdata.Rdata):
         self,
         rdclass: dns.rdataclass.RdataClass,
         rdtype: dns.rdatatype.RdataType,
-        strings: Iterable[Union[bytes, str]],
+        strings: Iterable[bytes | str],
     ):
         """Initialize a TXT-like rdata.
 
@@ -51,17 +53,19 @@ class TXTBase(dns.rdata.Rdata):
         self.strings: Tuple[bytes] = self._as_tuple(
             strings, lambda x: self._as_bytes(x, True, 255)
         )
+        if len(self.strings) == 0:
+            raise ValueError("the list of strings must not be empty")
 
     def to_text(
         self,
-        origin: Optional[dns.name.Name] = None,
+        origin: dns.name.Name | None = None,
         relativize: bool = True,
-        **kw: Dict[str, Any]
+        **kw: Dict[str, Any],
     ) -> str:
         txt = ""
         prefix = ""
         for s in self.strings:
-            txt += '{}"{}"'.format(prefix, dns.rdata._escapify(s))
+            txt += f'{prefix}"{dns.rdata._escapify(s)}"'
             prefix = " "
         return txt
 
@@ -71,9 +75,9 @@ class TXTBase(dns.rdata.Rdata):
         rdclass: dns.rdataclass.RdataClass,
         rdtype: dns.rdatatype.RdataType,
         tok: dns.tokenizer.Tokenizer,
-        origin: Optional[dns.name.Name] = None,
+        origin: dns.name.Name | None = None,
         relativize: bool = True,
-        relativize_to: Optional[dns.name.Name] = None,
+        relativize_to: dns.name.Name | None = None,
     ) -> dns.rdata.Rdata:
         strings = []
         for token in tok.get_remaining():
@@ -93,10 +97,8 @@ class TXTBase(dns.rdata.Rdata):
 
     def _to_wire(self, file, compress=None, origin=None, canonicalize=False):
         for s in self.strings:
-            l = len(s)
-            assert l < 256
-            file.write(struct.pack("!B", l))
-            file.write(s)
+            with dns.renderer.prefixed_length(file, 1):
+                file.write(s)
 
     @classmethod
     def from_wire_parser(cls, rdclass, rdtype, parser, origin=None):
